--[[
Author: Starinnia
CPR is a combo points display addon based on Funkydude's BasicComboPoints
$Date: 2008-11-24 00:00:20 +0000 (Mon, 24 Nov 2008) $
$Revision: 7 $
]]

local _, class = UnitClass("player")
if class == "ROGUE" or class == "DRUID" then
	class = nil
else
	DisableAddOn("ComboPointsRedux")
	return
end

local basepath = "Interface\\Addons\\ComboPointsRedux\\Icons\\"
local GetComboPoints = GetComboPoints
local graphics = nil
local text = nil
local db
local defaults = {
	profile = {
		textpos = {},
		graphicpos = {},
		locked = false,
		font = "Friz Quadrata TT",
		scale = 1,
		fontsize = 12,
		spacing = 5,
		normColor = {1, 0.9, 0},
		fullColor = {1, 0, 0},
		textColor = {1, 1, 1},
		alpha = 1,
		icon = "square",
	}
}
	
local ComboPointsRedux = LibStub("AceAddon-3.0"):NewAddon("ComboPointsRedux", "AceConsole-3.0", "AceEvent-3.0")
local LSM = LibStub("LibSharedMedia-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("ComboPointsRedux")

local cprOps
local function getCPROps()
	if not cprOps then
		cprOps = {
			type = 'group',
			name = "ComboPointsRedux",
			args = {
				lock = {
					type = "toggle",
					name = L["Lock"],
					get = function() return db.locked end,
					set = function()
						if not db.locked then
							graphics:SetBackdropColor(1,1,1,0)
							text:SetBackdropColor(1,1,1,0)
							graphics:EnableMouse(false)
							graphics:SetMovable(false)
							text:EnableMouse(false)
							text:SetMovable(false)
							db.locked = true
						else
							graphics:SetBackdropColor(1,1,1,1)
							text:SetBackdropColor(1,1,1,1)
							graphics:EnableMouse(true)
							graphics:SetMovable(true)
							text:EnableMouse(true)
							text:SetMovable(true)
							db.locked = false
						end
					end,
					order = 1,
				},
				font = {
					type = "select",
					name = L["Font"],
					get = function() return db.font end,
					set = function(_, v)
						db.font = v
						text.t:SetFont(LSM:Fetch('font', v), db.fontsize)
					end,
					dialogControl = "LSM30_Font",
					values = AceGUIWidgetLSMlists.font,
					order = 2,
				},
				fontsize = {
					type = "range",
					name = L["Font Size"],
					get = function() return db.fontsize end,
					set = function(_, v)
						db.fontsize = v
						text.t:SetFont(LSM:Fetch('font', db.font), v)
					end,
					min = 1,
					max = 36,
					step = 1,
					width = 'full',
					order = 3,
				},
				scale = {
					type = "range",
					name = L["Scale"],
					get = function() return db.scale end,
					set = function(_, scale)
						db.scale = scale
						for i = 1, 5 do
							graphics.points[i]:SetHeight(25*scale)
							graphics.points[i]:SetWidth(25*scale)
						end
						text:SetHeight(50*scale)
						text:SetWidth(50*scale)
						graphics:SetHeight(25*scale)
						graphics:SetWidth(145*scale)
						graphics.points[2]:SetPoint("LEFT", graphics.points[1], "RIGHT", db.spacing*scale, 0)
						graphics.points[3]:SetPoint("LEFT", graphics.points[2], "RIGHT", db.spacing*scale, 0)
						graphics.points[4]:SetPoint("LEFT", graphics.points[3], "RIGHT", db.spacing*scale, 0)
						graphics.points[5]:SetPoint("LEFT", graphics.points[4], "RIGHT", db.spacing*scale, 0)
					end,
					min = .1,
					max = 1.5,
					step = .01,
					width = 'full',
					order = 4,
				},
				spacing = {
					type = 'range',
					name = L["Spacing"],
					get = function() return db.spacing end,
					set = function(_, v)
						db.spacing = v
						graphics.points[2]:SetPoint("LEFT", graphics.points[1], "RIGHT", v*db.scale, 0)
						graphics.points[3]:SetPoint("LEFT", graphics.points[2], "RIGHT", v*db.scale, 0)
						graphics.points[4]:SetPoint("LEFT", graphics.points[3], "RIGHT", v*db.scale, 0)
						graphics.points[5]:SetPoint("LEFT", graphics.points[4], "RIGHT", v*db.scale, 0)
					end,
					min = -2,
					max = 10,
					step = 1,
					width = 'full',
					order = 5,
				},
				alpha = {
					type = 'range',
					name = L["Alpha"],
					get = function() return db.alpha end,
					set = function(_, v)
						db.alpha = v
						text.t:SetAlpha(v)
						text.t:SetShadowColor(0, 0, 0, db.alpha)
						for i = 1, 5 do
							graphics.points[i]:SetAlpha(v)
						end
					end,
					min = 0,
					max = 1,
					step = .01,
					width = 'full',
					order = 6,
				},
				normalcolor = {
					type = 'color',
					name = L["Normal Color"],
					get = function() return unpack(db.normColor) end,
					set = function(_, r, g, b)
						db.normColor[1] = r
						db.normColor[2] = g
						db.normColor[3] = b
						ComboPointsRedux:ReDraw()
					end,
					order = 7,
				},
				fullcolor = {
					type = 'color',
					name = L["Five Point Color"],
					get = function() return unpack(db.fullColor) end,
					set = function(_, r, g, b)
						db.fullColor[1] = r
						db.fullColor[2] = g
						db.fullColor[3] = b
						ComboPointsRedux:ReDraw()
					end,
					order = 8,
				},
				textcolor = {
					type = 'color',
					name = L["Text Color"],
					get = function() return unpack(db.textColor) end,
					set = function(_, r, g, b)
						db.textColor[1] = r
						db.textColor[2] = g
						db.textColor[3] = b
						text.t:SetTextColor(r, g, b)
					end,
					order = 9,
				},
				icon = {
					type = 'select',
					name = L["Combo Point Icon"],
					get = function() return db.icon end,
					set = function(_, v)
						db.icon = v
						for i = 1, 5 do
							graphics.points[i].icon:SetTexture(basepath..db.icon)
						end
					end,
					values = {
						square = L["Square"],
						circle = L["Circle"],
						triangle = L["Triangle"],
					},
					order = 10,
				},
			},
		}
	end
	return cprOps
end

function ComboPointsRedux:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("CPRDB", defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Refresh")
	self.db.RegisterCallback(self, "OnProfileChanged", "Refresh")
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("ComboPointsRedux", getCPROps)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("ComboPointsRedux", "ComboPointsRedux")
	self:RegisterChatCommand("cpr", "OpenConfig", true, true)
end

function ComboPointsRedux:OnEnable()
	self:MakeTextFrame()
	self:MakeGraphicsFrame()
	
	self:RegisterEvent("UNIT_COMBO_POINTS", "ReDraw")
	self:RegisterEvent("PLAYER_TARGET_CHANGED", "ReDraw")
end

function ComboPointsRedux:Reset()
	text:ClearAllPoints()
	graphics:ClearAllPoints()
	
	text:SetPoint("CENTER", UIParent, "CENTER")
	graphics:SetPoint("CENTER", UIParent, "CENTER")
	
	for i = 1, 5 do
		graphics.points[i].icon:SetTexture(basepath.."square")
	end
end

function ComboPointsRedux:Refresh()
	text:ClearAllPoints()
	graphics:ClearAllPoints()
	
	if db.position.x then
		text:SetPoint(db.textpos.point, UIParent, db.textpos.anchor, db.textpos.x, db.textpos.y)
		graphics:SetPoint(db.graphicpos.point, UIParent, db.graphicpos.anchor, db.graphicpos.x, db.graphicpos.y)
	else
		text:SetPoint("CENTER", UIParent, "CENTER")
		graphics:SetPoint("CENTER", UIParent, "CENTER")
	end
	
	for i = 1, 5 do
		graphics.points[i].icon:SetTexture(basepath..db.icon)
	end
end

function ComboPointsRedux:OpenConfig()
	LibStub("AceConfigDialog-3.0"):SetDefaultSize("ComboPointsRedux", 500, 550)
	LibStub("AceConfigDialog-3.0"):Open("ComboPointsRedux")
end

function ComboPointsRedux:ReDraw()
	local points = GetComboPoints("player", "target")
	
	if points == 0 then
		for i = 1, 5 do
			graphics.points[i]:Hide()
			graphics.points[i].icon:SetVertexColor(unpack(db.normColor))
		end
		points = ""
	elseif points == 5 then
		for i = points, 1, -1 do
			graphics.points[i].icon:SetVertexColor(unpack(db.fullColor))
			graphics.points[i]:Show()
		end
	else
		for i = points, 1, -1 do
			graphics.points[i]:Show()
		end
	end
	
	text.t:SetText(points)
end

local bg = {bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",}
function ComboPointsRedux:MakeTextFrame()
	text = CreateFrame("FRAME", "CPRTEXTFRAME", UIParent)
	text:SetBackdrop(bg)
	text:SetFrameStrata("BACKGROUND")
	text:SetClampedToScreen(true)
	if db.textpos.x then
		text:SetPoint(db.textpos.point, UIParent, db.textpos.anchor, db.textpos.x, db.textpos.y)
	else
		text:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	end
	text:SetBackdropColor(1,1,1,0)
	text:SetWidth(50*db.scale)
	text:SetHeight(50*db.scale)
	
	text.t = text:CreateFontString(nil,"OVERLAY")
	text.t:SetPoint("CENTER", text, "CENTER")
	text.t:SetShadowColor(0, 0, 0, db.alpha)
	text.t:SetShadowOffset(1, -1)
	text.t:SetFont(LSM:Fetch("font", db.font), db.fontsize)
	text.t:SetTextColor(unpack(db.textColor))
	text.t:SetAlpha(db.alpha)
	
	if not db.locked then
		text:SetBackdropColor(1,1,1,1)
		text:EnableMouse(true)
		text:SetMovable(true)
	end
	
	text:RegisterForDrag("LeftButton")
	text:SetScript("OnDragStart", function() this:StartMoving() end)
	text:SetScript("OnDragStop", function()
		this:StopMovingOrSizing()
		local point, _, anchor, x, y = this:GetPoint()
		db.textpos.x = math.floor(x)
		db.textpos.y = math.floor(y)
		db.textpos.anchor = anchor
		db.textpos.point = point
	end)
	
	text:Show()
end

function ComboPointsRedux:MakeGraphicsFrame()
	graphics = CreateFrame("FRAME", "CPRGRAPHICSFRAME", UIParent)
	graphics:SetBackdrop(bg)
	graphics:SetFrameStrata("BACKGROUND")
	graphics:SetClampedToScreen(true)
	if db.graphicpos.x then
		graphics:SetPoint(db.graphicpos.point, UIParent, db.graphicpos.anchor, db.graphicpos.x, db.graphicpos.y)
	else
		graphics:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	end
	graphics:SetBackdropColor(1,1,1,0)
	graphics:SetWidth(145*db.scale)
	graphics:SetHeight(25*db.scale)
	
	if not db.locked then
		graphics:SetBackdropColor(1,1,1,1)
		graphics:EnableMouse(true)
		graphics:SetMovable(true)
	end
	
	graphics:RegisterForDrag("LeftButton")
	graphics:SetScript("OnDragStart", function() this:StartMoving() end)
	graphics:SetScript("OnDragStop", function()
		this:StopMovingOrSizing()
		local point, _, anchor, x, y = this:GetPoint()
		db.graphicpos.x = math.floor(x)
		db.graphicpos.y = math.floor(y)
		db.graphicpos.anchor = anchor
		db.graphicpos.point = point
	end)
	
	graphics.points = {}
	for i = 1, 5 do
		graphics.points[i] = CreateFrame("FRAME", nil, graphics)
		--graphics.points[i]:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8X8",})
		--graphics.points[i]:SetBackdropColor(unpack(db.normColor))
		graphics.points[i].icon = graphics.points[i]:CreateTexture(nil, "OVERLAY")
		graphics.points[i].icon:SetAllPoints(graphics.points[i])
		graphics.points[i].icon:SetTexture(basepath..db.icon)
		graphics.points[i].icon:SetVertexColor(unpack(db.normColor))
		graphics.points[i]:SetAlpha(db.alpha)
		graphics.points[i]:SetHeight(25*db.scale)
		graphics.points[i]:SetWidth(25*db.scale)
		graphics.points[i]:Hide()
	end
	
	graphics.points[1]:SetPoint("LEFT", graphics, "LEFT", 0, 0)
	graphics.points[2]:SetPoint("LEFT", graphics.points[1], "RIGHT", db.spacing*db.scale, 0)
	graphics.points[3]:SetPoint("LEFT", graphics.points[2], "RIGHT", db.spacing*db.scale, 0)
	graphics.points[4]:SetPoint("LEFT", graphics.points[3], "RIGHT", db.spacing*db.scale, 0)
	graphics.points[5]:SetPoint("LEFT", graphics.points[4], "RIGHT", db.spacing*db.scale, 0)
	
	graphics:Show()
end
